/*********************************************
This program was produced by the
CodeWizardAVR V1.24.0 Standard
Automatic Program Generator
 Copyright 1998-2003 HP InfoTech s.r.l.
http://www.hpinfotech.ro
e-mail:office@hpinfotech.ro

Project : PPE robot suiveur de ligne 
Version : 3
Date    : 01/05/2008
Author  : Mariano philippe, Lorrain, Herveau, Peras                            
Company : Lyce Pierre Emile Martin
1, Avenue de Gionne 18000 BOURGES                                
Comments: Programme du robot """"""""Ramasseur"""""""
avec modification des paramtres PWM sur le robot
et test des servos

Chip type           : ATmega8535
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

//************************************************************************************************
// Librairies  
// --------------------------------------------------------------------------------------------- 
#include <mega8535.h> 
#include <delay.h>
#include <stdio.h>       
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ssi.h>  		// Fonction d'affichage
#include "LibRobot.h" 		// Gestion clavier carte robot
//************************************************************************************************

// Alphanumeric LCD Module functions   
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>

//************************************************************************************************
// Dfinitions  divers
// ----------------------------------------------------------------------------------------------- 
#define ACO 	ACSR.5      // Sortie du comparateur analogique      

// Caractres spcifiques  afficher sur le LCD  
// ----------------------------------------------------------------------------------------------- 
#define Caractere_FlecheH	0 
#define Caractere_FlecheB	1

// Adresses de la CGRAM de l'afficheur LCD type Hitachi
// ----------------------------------------------------------------------------------------------- 
//@ : adresse 
#define Add_FlecheH_CGRAM 0x40 // @1 Caractre flche haut
#define Add_FlecheB_CGRAM 0x48 // @2 Caractre flche bas

// Signaux de commande des capteurs IR 
// ----------------------------------------------------------------------------------------------- 
// PA7 | PA6 | PA5 | PA4  | PA3 | PA2 | PA1 | PA0
// D3  | D2  | D1  | IRG  |IRCG | IRC |IRCD | IRD
#define CAPTEUR_IR 	PORTA     // PORTA partag entre Del_IR et Dels Visu

// Signaux de commande des moteurs 
// -----------------------------------------------------------------------------------------------  
#define CDE_MOTD	OCR1A       // Cde moteur Droit
#define CDE_MOTG	OCR1B	    // Cde moteur Gauche

// Signaux de commande des Servos
// ----------------------------------------------------------------------------------------------- 
#define CDE_SERVO1	PORTB.0
#define CDE_SERVO2	PORTB.1

// Dels affichage position robot par rapport  la ligne
// ----------------------------------------------------------------------------------------------- 
#define DELG    PORTB.4 	// Led D4 sur la carte Robot SI
#define DELCG   PORTA.7 	// Led D3   "	"	"
#define DELCD	PORTA.6 	// Led D2   "	"	"
#define DELD    PORTA.5 	// Led D1   "	"	"

//Pour info: Rapports cycliques alpha du signal de commande des moteurs 
// ----------------------------------------------------------------------------------------------- 
// OCR1x = 255*alpha
// 		OCR1x		 alpha 
#define _0PC	 0		 // 0% 
#define _5PC	 13              // 5% 
#define _10PC	 25              // 10%
#define _15PC	 38              // 15%
#define _20PC    51		 // 20%
#define _25PC	 64              // 25%
#define _30PC    76		 // 30%
#define _35PC	 89              // 35% 
#define _40PC    102		 // 40%
#define _45PC	 115             // 45%
#define _50PC    128             // 50%
#define _55PC	 140             // 55% 
#define _60PC    153             // 60%
#define _65PC	 166             // 65%
#define _70PC    179             // 70%
#define _75PC	 191             // 75%
#define _80PC    205             // 80%     
#define _85PC    219             // 85% 
#define _90PC    233             // 90% 
// ----------------------------------------------------------------------------------------------*/
//***************************************************************************************************

//***************************************************************************************************
// Constantes en eeprom
// -----------------------------------------------------------------------------------------------
// Table des valeurs de rglage PWM en eeprom  ( initialiser avec le fichier Robot2base.eep 
// lors de la premire mise en oeuvre
//*************************************************************************************************** 
//Organisation de la table		MG   MD	
eeprom unsigned char PWM_EEPROM[11][2];  	// Centre
				          	// Droite
				         	// Droite+
				    		// Droite2+
				    		// Droite3+
				    		// HorsLigneD
				    		// Gauche
				    		// Gauche+
				    		// Gauche2+
				    		// Gauche3+
				    		// HorsLigneG
//***************************************************************************************************
// Constantes en flash
// --------------------------------------------------------------------------------------------------
// Tables des caractres personnaliss  placer dans la mmoire CGRAM du LCD  
// --------------------------------------------------------------------------------------------------
const unsigned char flecheHaut[8] = {0x04, 0x0E, 0x15, 0x04, 0x04, 0x04, 0x04, 0x00};    // @1 CGRAM
const unsigned char flecheBas[8] = {0x04, 0x04, 0x04, 0x04, 0x15, 0x0E, 0x04, 0x00};     // @2 CGRAM
// --------------------------------------------------------------------------------------------------
// Tableau de structures des menus "Rglage des paramtres PWM"  
// --------------------------------------------------------------------------------------------------
flash struct Affichage_reglages {
		char ligne0[14];
		char ligne1[14];  
		} Tab_Aff_Reg[11]= {{"Centre MG  =","Centre MD  ="},
				    {"Droite MG  =","Droite MD  ="},
				    {"Droite+ MG =","Droite+ MD ="},
				    {"Droite2+ MG=","Droite2+ MD="},
				    {"Droite3+ MG=","Droite3+ MD="},
				    {"HorLigD MG =","HorLigD MD ="},
				    {"Gauche MG  =","Gauche MD  ="},
				    {"Gauche+ MG =","Gauche+ MD ="},
				    {"Gauche2+ MG=","Gauche2+ MD="},
				    {"Gauche3+ MG=","Gauche3+ MD="},
				    {"HorLigG MG =","HorLigG MD ="}};
// --------------------------------------------------------------------------------------------------
// Table de transcodage %PWM --> OCR1x
//--------------------------------------------------------------------------------------------------
const unsigned char Table_Calcul_OCR1x[19][2]={{0,0},{5,13},{10,25},{15,38},{20,51},{25,64},{30,76},
{35,89},{40,102},{45,115},{50,128},{55,140},{60,153},{65,166},{70,179},{75,191},{80,205},{85,219},
{90,233}}; 		
//***************************************************************************************************
 
//************************************************************************************************
// Prototype des fonctions  
// ----------------------------------------------------------------------------------------------- 
void Arret_Moteurs(void);	// Dsactivation des moteurs
void Active_Timer(void);    	// Activation des moteurs
void Mode_Run(void);        	// Suivi de ligne etc... 
void Prise_Balle(void);		// Squence de prise de la balle
//************************************************************************************************

//************************************************************************************************
// Declare your global variables here 
// ----------------------------------------------------------------------------------------------- 
// type		nom		Commentaires
//------------------------------------------------------------------------------------------------ 
// Etats du graphe des transitions
// -----------------------------------------------------------------------------------------------
enum TMenu {Menu_Reglages, Menu_Servo, Menu_Run, Reglage_PWM, Activation, Affiche_Reglages,Cde_Servo
 	} Menu=Menu_Reglages,Menu_1=Menu_Run;// Etats du graphe tests         
// -----------------------------------------------------------------------------------------------
// Commandes des moteurs
// -----------------------------------------------------------------------------------------------
enum Etat_Commande_Moteur {DEPART,CDE_CENTRE, CDE_CENTRE2, CDE_CENTRE3, CDE_DROITE, CDE_DROITE_PLUS, 
CDE_DROITE_2PLUS, CDE_DROITE_3PLUS, CDE_GAUCHE, CDE_GAUCHE_PLUS,CDE_GAUCHE_2PLUS,CDE_GAUCHE_3PLUS,
CDE_HORS_LIGNE_GAUCHE, CDE_HORS_LIGNE_DROITE } EtatCdeMot = DEPART;
// -----------------------------------------------------------------------------------------------
// Positions du robot par rapport  la ligne 
// -----------------------------------------------------------------------------------------------
enum Etat_Position_Robot {HORS_LIGNE, GAUCHE_3PLUS, GAUCHE_2PLUS = 3, CENTRE_1, GAUCHE_PLUS=7, 
CENTRE_3=14, GAUCHE, DROITE_3PLUS, DROITE_2PLUS=24, DROITE_PLUS=28, DROITE = 30,CENTRE_5} 
Position_Robot = HORS_LIGNE; 
//***************************************************************************************************
// Dtection de la cible => INT0 
// -----------------------------------------------------------------------------------------------
bit Cible=0;

//***************************************************************************************************
// SP dtection cible 
// -----------------------------------------------------------------------------------------------
// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
// Dsactivation de l'INT0
GICR=0x00; // Dsactivation INT0 (bit6 de GICR)
Arret_Moteurs();  // Arrt du robot
Cible = 1;
}  

//***************************************************************************************************
// Programme principal
//***************************************************************************************************
void main(void)
{ // Dbut du main
//------------------------------------------------------------------------------------------------
// Declare your local variables here
// type		nom				Commentaires
//------------------------------------------------------------------------------------------------
unsigned char BP;			// Etat des boutons-poussoir 
unsigned char i;                        // Compteur d'usage gnral
unsigned char rcycl, rcycl_1;           // Saisie du rapport cyclique lors d'une modification manuelle
unsigned char NumServo=1, NumServo_1=1;	// Saisie du numro du servo
unsigned char decompte=5, decompte_1=5;	// Temporisation avant le passage en mode RUN
char i_n=0, i_n_1=1; 			// Compteurs: position dans le tableau des structures 
					// d'affichage
char display_buffer_ligne0[17]; 	// Tampon ligne 0 de l'afficheur LCD
char display_buffer_ligne1[17]; 	// Tampon ligne 1 de l'afficheur LCD
char pourcentage0[2], pourcentage1[2];  // Affichage pourcentage
//------------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------------
// Initialisation des priphriques
//------------------------------------------------------------------------------------------------                                        
// Input/Output Ports initialization
// Port A initialization
// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=Out Func6=Out Func7=Out 
// State0=1 State1=1 State2=1 State3=1 State4=1 State5=1 State6=1 State7=1 
		// Gestion des E_IR et des DEls (sauf D4)
PORTA=0xFF; 	// Dsactivation des EIR et des Dels  la mise sous tension
DDRA=0xFF;	// Port en sortie

// Port B initialization
// Func0=Out Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=1 State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
		// Gestion des BP (sauf S4), D4, signal RIR, seuil capteur
PORTB=0x10;     // Del4 reli  PB4 (dsactive  la mise sous tension)
DDRB=0x13;      // CDE_SERVO1 = 0 et CDE_SERVO2= 0 => position repos

// Port C initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
            // Gestion de l'afficheur LCD
PORTC=0x00; // Gre l'affichage, initialis par la fonction lcd_init(16);
DDRC=0x00;    

// Port D initialization
// Func0=In Func1=In Func2=In Func3=Out Func4=In Func5=In Func6=Out Func7=In
// State0=T State1=T State2=T State3=0 State4=T State5=T State6=0 State7=T
		// Capteur magntique, moteurs, S4 et liaison srie
PORTD=0x04;     // Capteur magntique reli au 0V sur PD2
DDRD=0x30; 	// Broches PWM moteurs droit et gauche en sortie, initialise  0

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 8000,000 kHz
// Mode: Fast PWM top=00FFh
// OC1A output: Non-Inv.
// OC1B output: Non-Inv.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0xA1;
TCCR1B=0x09;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x02;
MCUCSR=0x00;
GIFR=0x40;

// Analog Comparator initialization
// Analog Comparator: On
// Analog Comparator Input Capture by Timer/Counter 1: Off
// Analog Comparator Output: Off
ACSR=0x00;
SFIOR=0x00;

// LCD module initialization
lcd_init(16);

// Chargement initial de la mmoire CGRAM de l'afficheur avec deux 
// caractres personaliss 
for (i=0;i<8;i++)
{
lcd_write_byte(Add_FlecheH_CGRAM + i, flecheHaut[i]); // @1 Caractre flche haut
lcd_write_byte(Add_FlecheB_CGRAM + i, flecheBas[i]); // @2 Caractre flche bas
}

//Initialisation de la position des Servos
CDE_SERVO1 = 0;
CDE_SERVO2 = 0;

//------------------------------------------------------------------------------------------------
 
// ------------------------------------Message d'accueil-------------------------------------------
lcd_clear();
sprintf(display_buffer_ligne0,"******RBOT******");
sprintf(display_buffer_ligne1,"---PEM BOURGES--");
Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);   
delay_ms(3000);
//------------------------------------------------------------------------------------------------
while (1)
{    
   BP = Lire_BP_Robot();
   
   switch (Menu)
   {// Dbut du switch principal
      
// -------------------------------------Dbut Menus droulants-------------------------------------
      case Menu_Reglages : if (Menu_1 != Menu)
                       {
                        sprintf(display_buffer_ligne0," Menu Reglages ");
                    	sprintf(display_buffer_ligne1,"[Etr]>Oui   suiv");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(11,1);	// Affichage flecheHaut
			lcd_putchar(Caractere_FlecheH);	
			lcd_gotoxy(10,1);	// Affichage flecheBas
			lcd_putchar(Caractere_FlecheB);
                       }
                      
                         switch(BP)
                         {                                  
                      	   case S4: Menu = Affiche_Reglages;  break; // touche Enter
                      	   case S2: Menu = Menu_Run;  break; // touche flche haut
                      	   case S1: Menu = Menu_Servo;  break; // touche flche bas
                      	   default: Menu = Menu_Reglages;
                      	 }
                       
       		         Menu_1 = Menu_Reglages;
       break;  
       
       case Menu_Servo : if (Menu_1 != Menu)
                       {
                        sprintf(display_buffer_ligne0,"--Test Servos?--");
                    	sprintf(display_buffer_ligne1,"[Etr]>Oui   suiv");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(11,1);	// Affichage flecheHaut
			lcd_putchar(Caractere_FlecheH);	
			lcd_gotoxy(10,1);	// Affichage flecheBas
			lcd_putchar(Caractere_FlecheB);
                       }
                      
                         switch(BP)
                         {                                  
                      	   case S4: Menu = Cde_Servo;  break; // touche Enter
                      	   case S2: Menu = Menu_Reglages;  break; // touche flche haut
                      	   case S1: Menu = Menu_Run;  break; // touche flche bas
                      	   default: Menu = Menu_Servo;
                      	 }
                       
       		         Menu_1 = Menu_Servo;
       break; 
       case Menu_Run : if (Menu_1 != Menu)
                       {
                        sprintf(display_buffer_ligne0,"Activer robot ? ");
                    	sprintf(display_buffer_ligne1,"[Etr]>Oui   suiv");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(11,1);	// Affichage flecheHaut
			lcd_putchar(Caractere_FlecheH);	
			lcd_gotoxy(10,1);	// Affichage flecheBas
			lcd_putchar(Caractere_FlecheB);
                       }
                      
                        switch(BP)
                         {                                  
                      	   case S4: Menu = Activation; break; // touche Enter
                      	   case S2: Menu = Menu_Servo; break; // touche flche haut
                      	   case S1: Menu = Menu_Reglages; break; // touche flche bas
                      	   default: Menu = Menu_Run;
                      	 }
                       
       		        Menu_1 = Menu_Run;
       break;   
       
       case Affiche_Reglages : if ((Menu_1 != Menu) || (i_n_1 != i_n))
                       {
                        if (fmod(i_n,2)==0){ // si i_n pair
                        	sprintf(display_buffer_ligne0,Tab_Aff_Reg[i_n/2].ligne0);
                        	itoa(PWM_EEPROM[i_n/2][0],pourcentage0);
                        	strcat(display_buffer_ligne0,pourcentage0);
                       		sprintf(display_buffer_ligne1,Tab_Aff_Reg[i_n/2].ligne1);
                       		itoa(PWM_EEPROM[i_n/2][1],pourcentage1);
                       	        strcat(display_buffer_ligne1,pourcentage1);
                       		}
                       		else { // si i_n impair
                        	sprintf(display_buffer_ligne0,Tab_Aff_Reg[i_n/2].ligne1);
                        	itoa(PWM_EEPROM[i_n/2][1],pourcentage0);
                                strcat(display_buffer_ligne0,pourcentage0);
                       		sprintf(display_buffer_ligne1,Tab_Aff_Reg[i_n/2+1].ligne0);
                       		itoa(PWM_EEPROM[i_n/2+1][0],pourcentage1);
                       	 	strcat(display_buffer_ligne1,pourcentage1);
                       		}
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(14,0);
                    	lcd_putchar('%');
                    	lcd_gotoxy(15,0);	// Affichage flecheHaut
			lcd_putchar(Caractere_FlecheH);
			lcd_gotoxy(14,1);
                    	lcd_putchar('%');	
			lcd_gotoxy(15,1);	// Affichage flecheBas
			lcd_putchar(Caractere_FlecheB);
			i_n_1 = i_n;
                       }
                      
                         switch(BP)
                         {                                  
                      	   case S4:  if (fmod(i_n,2)==0) // si i_n pair
                      	                 rcycl = PWM_EEPROM[i_n/2][0];
                      	   	     else     
                      	   	         rcycl = PWM_EEPROM[i_n/2][1];
                      	   	         
                      	             Menu = Reglage_PWM;  
                      	   break;  
                      	   case S3: Menu = Menu_Reglages; break;
                      	   case S1: if (i_n < 20) i_n++; else i_n=0; break; 
                      	   case S2: if (i_n > 0) i_n--; else i_n=20; break; 
                      	   default: Menu = Affiche_Reglages;
                      	 }
                       
       		       Menu_1 = Affiche_Reglages;
       break;   
         
       case Reglage_PWM : if ((Menu_1 != Menu)|| (rcycl_1 != rcycl))
                       { 
                        if (fmod(i_n,2)==0){ // si i_n pair
                        	sprintf(display_buffer_ligne0,Tab_Aff_Reg[i_n/2].ligne0);
                        	itoa(rcycl,pourcentage0);
                        	strcat(display_buffer_ligne0,pourcentage0);}
                        	else{
                        	sprintf(display_buffer_ligne0,Tab_Aff_Reg[i_n/2].ligne1);
                        	itoa(rcycl,pourcentage0);
                                strcat(display_buffer_ligne0,pourcentage0);}
                    	sprintf(display_buffer_ligne1,"Etr>Memo ECH>Q ");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	lcd_gotoxy(15,0);
			lcd_putchar('+');
			lcd_gotoxy(15,1);
			lcd_putchar('-');
			rcycl_1 = rcycl;
                       }
                      
                         switch(BP)
                         {
                           case S4: if (fmod(i_n,2)==0) // si i_n pair
                      	                  PWM_EEPROM[i_n/2][0]=rcycl;
                      	   	     else     
                      	   	          PWM_EEPROM[i_n/2][1]=rcycl; 
                      	   	     Menu = Affiche_Reglages;     
                      	   break;                                  
                      	   case S3: Menu = Affiche_Reglages;  break; // touche Enter
                      	   case S2: if (rcycl < 95) rcycl = rcycl+5; else rcycl = 95; break; 
                      	   case S1: if (rcycl > 5) rcycl = rcycl-5; else rcycl = 0; break; 
                      	   default: Menu = Reglage_PWM;
                      	 }
                       
       		       Menu_1 = Reglage_PWM;
       break;   
       
       case Activation : if ((Menu_1 != Menu) || (decompte != decompte_1))
                       {
                        sprintf(display_buffer_ligne0,"Depart dans %-us",decompte);
                    	sprintf(display_buffer_ligne1,"[Ech]>Desactiver");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	decompte_1 = decompte;
                       }
                       
                     if (BP==S3) {
                                   decompte=8;
                                   Menu=Menu_Reglages;
                                   break;
                     		 }
                       		  else if(decompte == 0) 
                     		       		Mode_Run();
                     delay_ms(1000); decompte--;
                     Menu_1 = Activation;
      break; 
      
      case Cde_Servo : if ((Menu_1 != Menu) || (NumServo != NumServo_1))
                       {
                        sprintf(display_buffer_ligne0,"Commande Servo%-u ",NumServo);
                    	sprintf(display_buffer_ligne1,"[Etr]>Ch [Ech]>Q");
                    	Affiche_LCD(display_buffer_ligne0,display_buffer_ligne1);
                    	NumServo_1 = NumServo;
                       }
                       
                     switch(BP)
                         {                                  
                      	   case S4: if (NumServo==2) NumServo=1;
                      	   		else if (NumServo==1) NumServo=2;		
                      	   break;  
                      	   case S3: Menu = Menu_Servo; break;
                      	   case S2: if (NumServo==1) {CDE_SERVO1=1; DELD = 0;}
                      	   		else if  (NumServo==2) {CDE_SERVO2=1; DELCD = 0;}   
                      	   break;
                      	   case S1: if (NumServo==1) {CDE_SERVO1=0;DELD = 1;}
                      	   		else if  (NumServo==2) {CDE_SERVO2=0; DELCD = 1;}
                      	   break; 
                      	   default: Menu = Cde_Servo;
                      	 }
                     Menu_1 = Cde_Servo;
      break;           
 // -------------------------------------Fin Menus droulants------------------------------------------------       
  } // Fin du switch principal
 } // Fin du while(1)
} // fin du main     


//***************************************************************************************************
// ************************************ Sous-programmes ********************************************
//***************************************************************************************************
// 	Mode_Run() 
//***************************************************************************************************
// Attention : on sort du menu Mode_Run par une coupure d'alimentation
//***************************************************************************************************
void Mode_Run()
{
// ----------------------------------------------------------------------------------------------- 
// Variables locale  Mode_Run()
// type		nom		Commentaires
// ----------------------------------------------------------------------------------------------- 
unsigned char i,j,k;            // Compteurs de boucle  
unsigned char PWM[11][2];	// Table charge avec les coefficients
				// PWM de rglage des moteurs
    				// issus de la table PWMEEPROM
unsigned char Cde_Del_IR;	// Permet de multiplexer la commande des dels IR
unsigned int Phase_Arret = 50;	// Compteur de boucle : Autorise la mesure de la position du robot
				// pendant 200ms aprs l'arrt des moteurs devant la cible 
// ----------------------------------------------------------------------------------------------- 
// -----------------------------------------------------------------------------------------------                                       
// Initialisation du mode Run: Chargement de PWM[][] alec les paramtres en EEPROM
// i,k: compteur ligne, j compteur colonne
for (i=0;i<11;i++)
 { 
  for (j=0;j<2;j++)
   { 
    for (k=0;k<19;k++)
     {
       if (PWM_EEPROM[i][j] == Table_Calcul_OCR1x[k][0])
         {   
	    PWM[i][j] = Table_Calcul_OCR1x[k][1];
	    break;
	  }
     }
    }
  } 
  
// Global enable interrupts
#asm("sei") 

// Fin initialisation
// ----------------------------------------------------------------------------------------------- 

while (1)
    {// dbut Mode_Run() 
// ----------------------------------------------------------------------------------------------- 
// Phase d'arrt devant la cible: Mesure de la position et volution du graphe pendant 
// [4.3*10E-3*Phase_Arret] puis lancement du sous-programme Prise_Balle()
// ----------------------------------------------------------------------------------------------- 
       if (Cible == 1) // Cible passe  1 lorsque le robot est devant la cible 
        {
          if (Phase_Arret == 0)
                Prise_Balle(); // Cible <- dans INT0
                else 
                  Phase_Arret--;
        }
// -----------------------------------------------------------------------------------------------     
// Fonction acqurir
// -----------------------------------------------------------------------------------------------          
      Position_Robot = HORS_LIGNE; // Initialisation  0	
      Cde_Del_IR = 1; 
 
      for (i=0;i<5;i++)  // Dbut Acquisition de la position du robot par rapport  la ligne
       {
        CAPTEUR_IR = ~Cde_Del_IR;
      	delay_us(20);	// Activation d'un capteur  ajuster   (19s par dfaut)
        if (ACO == 1) Position_Robot = Position_Robot | Cde_Del_IR;
        CAPTEUR_IR = 0xFF; DELG = 1;
        Cde_Del_IR = Cde_Del_IR << 1 ;
        delay_us(700);	//  ajuster   (540s par dfaut)
       } 
       			// Fin Acquisition de la position du robot par rapport  la ligne
// ----------------------------------------------------------------------------------------------- 
// Position_Robot  	Commentaires                                                         
// 	0		Hors_Ligne          
// 	1		Gauche3+            
// 	3 		Gauche2+
//	4		Centre1 (1 capteur IR sur la ligne) 	
// 	7		Gauche+ 
//	14		Centre3 (3 capteurs IR sur la ligne)
//	15		Gauche
//	16		Droite3+
//	24		Droite2+			
//	28		Droite+ 
//	30		Droite
// 	31 		Centre5 (5 capteurs IR sur la ligne)
                 
// ----------------------------------------------------------------------------------------------- 
// Fonction Visualiser (Visualisation des capteurs sur la ligne)
// ----------------------------------------------------------------------------------------------- 
switch (Position_Robot)              
       { 
// Position_Robot	Cde des Del
// -----------------------------------------------------------------------------------------------      
       //        DLG  |   DLCG |   DLCD |   DLD 
// ----------------------------------------------------------------------------------------------- 
	case 0:  DELG=1; DELCG=1; DELCD=1; DELD=1; break;// Hors ligne
 	case 1:  DELG=1; DELCG=1; DELCD=1; DELD=0; break;// Gauche3+
  	case 3:  DELG=1; DELCG=1; DELCD=0; DELD=0; break;// Gauche2+
   	case 4:  DELG=1; DELCG=1; DELCD=1; DELD=1; break;// Centre1 non visualis
        case 7:  DELG=1; DELCG=0; DELCD=0; DELD=0; break;// Gauche+  
        case 14: DELG=1; DELCG=0; DELCD=0; DELD=1; break;// Centre3 
        case 15: DELG=1; DELCG=0; DELCD=0; DELD=0; break;// Gauche 
        case 16: DELG=0; DELCG=1; DELCD=1; DELD=1; break;// Droite3+
        case 24: DELG=0; DELCG=0; DELCD=1; DELD=1; break;// Droite2+  
        case 28: DELG=0; DELCG=0; DELCD=0; DELD=1; break;// Droite+ 
        case 30: DELG=0; DELCG=0; DELCD=0; DELD=1; break;// Droite 
        case 31: DELG=0; DELCG=0; DELCD=0; DELD=0; break;// Centre5
       }   
// ----------------------------------------------------------------------------------------------- 
// Fonction Traiter
// -----------------------------------------------------------------------------------------------      
      switch (EtatCdeMot)              
       { // Dbut switch (EtatCdeMot)                                 
        case DEPART: Arret_Moteurs(); 
        		if ((Position_Robot == CENTRE_3)||(Position_Robot == CENTRE_5))
        		    { 
        		      //delay_ms(5000);
        		      Active_Timer();
        		      EtatCdeMot = CDE_CENTRE;
        		    }
        		    else  EtatCdeMot = DEPART;
        break;   
        
        case CDE_CENTRE : CDE_MOTG = PWM[0][0]; CDE_MOTD = PWM[0][1];
                          if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE;
                          	else if (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
                          		else EtatCdeMot = CDE_CENTRE;
        break;  
        
        
        case CDE_DROITE:CDE_MOTG = PWM[1][0]; CDE_MOTD = PWM[1][1]; 
                         if (Position_Robot == CENTRE_5 ) EtatCdeMot = CDE_CENTRE;
                          	else if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS;
                          		else EtatCdeMot = CDE_DROITE;
        break;   
        
        case CDE_DROITE_PLUS:CDE_MOTG = PWM[2][0]; CDE_MOTD = PWM[2][1];
        		if (Position_Robot == GAUCHE) EtatCdeMot = CDE_DROITE;
                             else if (Position_Robot == GAUCHE_2PLUS) EtatCdeMot = CDE_DROITE_2PLUS;
                        		else EtatCdeMot = CDE_DROITE_PLUS;
        break;  
        
        case CDE_DROITE_2PLUS: CDE_MOTG = PWM[3][0]; CDE_MOTD = PWM[3][1];
        		if (Position_Robot == GAUCHE_PLUS) EtatCdeMot = CDE_DROITE_PLUS; 
        	       		else if (Position_Robot == GAUCHE_3PLUS) EtatCdeMot = CDE_DROITE_3PLUS;
        				else EtatCdeMot = CDE_DROITE_2PLUS;
        break;
             
        case CDE_DROITE_3PLUS: CDE_MOTG = PWM[4][0]; CDE_MOTD = PWM[4][1];
        		if (Position_Robot == GAUCHE_2PLUS) EtatCdeMot = CDE_DROITE_2PLUS; 
        	       		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = CDE_HORS_LIGNE_DROITE;
        				else EtatCdeMot = CDE_DROITE_3PLUS;
        break;
        
        case CDE_HORS_LIGNE_DROITE : CDE_MOTG = PWM[5][0]; CDE_MOTD = PWM[5][1];
        		if (Position_Robot == GAUCHE_3PLUS) EtatCdeMot = CDE_DROITE_3PLUS;
        				else EtatCdeMot = CDE_HORS_LIGNE_DROITE;
        break;
        			  			  
        case CDE_GAUCHE: CDE_MOTG = PWM[6][0]; CDE_MOTD = PWM[6][1]; 
                       if (Position_Robot == CENTRE_5) EtatCdeMot = CDE_CENTRE;
                          	else if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS;
                          		else EtatCdeMot = CDE_GAUCHE;
        break;  
        
        case CDE_GAUCHE_PLUS: CDE_MOTG = PWM[7][0]; CDE_MOTD = PWM[7][1]; 
                      if  (Position_Robot == DROITE) EtatCdeMot = CDE_GAUCHE;
        	              else if (Position_Robot == DROITE_2PLUS) EtatCdeMot = CDE_GAUCHE_2PLUS;
                               		else EtatCdeMot = CDE_GAUCHE_PLUS;
        break; 
        
        case CDE_GAUCHE_2PLUS: CDE_MOTG = PWM[8][0]; CDE_MOTD = PWM[8][1];
        		if (Position_Robot == DROITE_PLUS) EtatCdeMot = CDE_GAUCHE_PLUS;
        		        else if (Position_Robot == DROITE_3PLUS) EtatCdeMot = CDE_GAUCHE_3PLUS;
        				else EtatCdeMot = CDE_GAUCHE_2PLUS;
        break;
        
        case CDE_GAUCHE_3PLUS: CDE_MOTG = PWM[9][0]; CDE_MOTD = PWM[9][1];
        		if (Position_Robot == DROITE_2PLUS) EtatCdeMot = CDE_GAUCHE_2PLUS;
        	       		else if (Position_Robot == HORS_LIGNE) EtatCdeMot = CDE_HORS_LIGNE_GAUCHE; 
        				else EtatCdeMot = CDE_GAUCHE_3PLUS;
        break;
        
        case CDE_HORS_LIGNE_GAUCHE: CDE_MOTG = PWM[10][0]; CDE_MOTD = PWM[10][1];
        		if (Position_Robot == DROITE_3PLUS) EtatCdeMot = CDE_GAUCHE_3PLUS;
        				else EtatCdeMot = CDE_HORS_LIGNE_GAUCHE;
        break;
        
        default : EtatCdeMot = CDE_CENTRE; 
        break;
         
       }// Fin switch (EtatCdeMot)        
// ----------------------------------------------------------------------------------------  
// Fin Traiter 

       delay_us(704); //  ajuster   (704s  par dfaut) 		
    }
} // fin Mode_Run();
//*************************************************************************************************** 

//***************************************************************************************************
// 			Arrt des moteurs 
//***************************************************************************************************
void Arret_Moteurs()
{
      TCCR1A = 0x00; // Arrt PWM 
      TCCR1B = 0x00; 
      PORTD=0x00;     // Arrt Moteur 
} 
//*************************************************************************************************** 

//***************************************************************************************************
// 			Activation des moteurs
//***************************************************************************************************
void Active_Timer()
{
      TCCR1A=0xA1;
      TCCR1B=0x09;
} 
//***************************************************************************************************  

//***************************************************************************************************  
// 			Squence de prise de la balle
//*************************************************************************************************** 
// Le robot est arrt dans le SP d'INT0
void Prise_Balle()
{
Cible = 0; // Dsactivation de la dtection de la cible

// Commande du bras (PB0 initialement en sortie et PB0=1)
// ========================================================
CDE_SERVO1 = 1; // Demande sortie du bras
delay_ms(1000);

// Commande de la main
//---------------------------------------------------------
CDE_SERVO2 = 1; // Activation Main
delay_ms(1000);

CDE_SERVO2 = 0; // Dsactivation Main (Economie nergie !)
delay_ms(10);

CDE_SERVO1 = 0;  // Demande rentre du bras
// =======================================================

// Marche Robot
Active_Timer();  

}